@@ -14,7 +14,7 @@ sub section {
 sub incorrect {
   my $expr = pop ;
   print "Incorrect expression $expr...\n" ;
-  system "$liquidsoap '$expr'" ;
+  system "$liquidsoap '$expr' >/dev/null 2>&1" ;
   die unless (($?>>8)==1) ;
   print "\n" ;
 }
@@ -22,11 +22,16 @@ sub incorrect {
 sub correct {
   my $expr = pop ;
   print "Correct expression $expr...\n" ;
-  system "$liquidsoap -i '$expr'" ;
+  system "$liquidsoap -i '$expr' >/dev/null 2>&1" ;
   die unless (($?>>8)==0) ;
   print "\n";
 }
 
+section("LISTS");
+incorrect('ignore([4,"x"])');
+correct('ignore([input.harbor("foo"), sine()])');
+correct('ignore([sine(), input.harbor("foo")])');
+
 section("BASIC");
 incorrect('[1]==["1"]');
 incorrect('1==["1"]');
@@ -45,7 +50,7 @@ correct('ignore(((blank():source(1,1,1)):source(*,*,*)))');
 incorrect('((blank():source(*,*,*)):source(1,1,1))');
 # Next one requires the inference of a subtype (fixed vs. variable arity)
 correct('ignore(audio_to_stereo(add([])))');
-correct('ignore((blank():source(audio=1,video=1,midi=1))');
+correct('ignore((blank():source(audio=1,video=1,midi=1)))');
 correct('ignore((blank():source(*+1,0,0)))');
 correct('ignore((blank():source(1+*,0,0)))');
 
