@@ -20,7 +20,7 @@
 
  *****************************************************************************)
 
- (** Samplerate converter using libsamplerate *)
+(** Samplerate converter using libsamplerate *)
 
 let samplerate_conf = 
   Dtools.Conf.void ~p:(Audio_converter.Samplerate.samplerate_conf#plug "libsamplerate") 
@@ -31,11 +31,11 @@ let quality_conf =
   Dtools.Conf.string ~p:(samplerate_conf#plug "quality")
     "Resampling quality" ~d:"fast"
     ~comments:["Resampling quality, one of: \
-                \"best\", \
-                \"medium\", \
-                \"fast\", \
-                \"zero_order\", \
-                \"linear\". Refer to ocaml-samplerate for details."]
+                `\"best\"`, \
+                `\"medium\"`, \
+                `\"fast\"`, \
+                `\"zero_order\"` or \
+                `\"linear\"`. Refer to ocaml-samplerate for details."]
 
 let quality_of_string v = 
   match v with
@@ -44,7 +44,7 @@ let quality_of_string v =
     | "fast" -> Samplerate.Conv_fastest
     | "zero_order" -> Samplerate.Conv_zero_order_hold
     | "linear" -> Samplerate.Conv_linear
-    | _ -> raise (Lang.Invalid_value 
+    | _ -> raise (Lang_errors.Invalid_value 
                     (Lang.string v,
                       "libsamplerate quality must be one of: \
                        \"best\", \
