@@ -20,9 +20,29 @@
 
  *****************************************************************************)
 
-  (** Native audio converters *)
+(** Native audio converters *)
 
-(* TODO: setting to be able to choose linear resampling. *)
-let samplerate_converter () x = Audio.Mono.resample x
+let samplerate_conf =
+  Dtools.Conf.void ~p:(Audio_converter.Samplerate.samplerate_conf#plug "native")
+    "Native samplerate conversion settings"
+    ~comments:["Options related to native samplerate conversion."]
+
+let quality_conf =
+  Dtools.Conf.string ~p:(samplerate_conf#plug "quality")
+    "Resampling quality" ~d:"linear"
+    ~comments:["Resampling quality: either \"nearest\" or \"linear\"."]
+
+let quality_of_string = function
+  | "nearest" -> `Nearest
+  | "linear" -> `Linear
+  | s ->
+    raise
+      (Lang_errors.Invalid_value
+         (Lang.string s,
+          "Native resampling quality must either be \"nearest\" or \"linear\"."))
+
+let samplerate_converter () =
+  let mode = quality_of_string quality_conf#get in
+  Audio.Mono.resample ~mode
 
 let () = Audio_converter.Samplerate.converters#register "native" samplerate_converter
