@@ -22,8 +22,6 @@
 
 (** Decode and read metadatas of AAC files. *)
 
-open Dtools
-
 let error_translator =
   function
     | Faad.Error x ->
@@ -140,12 +138,12 @@ let create_decoder input =
 end
 
 let aac_mime_types =
-  Conf.list ~p:(Decoder.conf_mime_types#plug "aac")
+  Dtools.Conf.list ~p:(Decoder.conf_mime_types#plug "aac")
     "Mime-types used for guessing AAC format"
     ~d:["audio/aac"; "audio/aacp"; "audio/x-hx-aac-adts"]
 
 let aac_file_extensions =
-  Conf.list ~p:(Decoder.conf_file_extensions#plug "aac")
+  Dtools.Conf.list ~p:(Decoder.conf_file_extensions#plug "aac")
     "File extensions used for guessing AAC format"
     ~d:["aac"]
 
@@ -169,7 +167,7 @@ let get_type filename =
         let n = Unix.read fd aacbuf 0 aacbuflen in
         Faad.init dec aacbuf 0 n
       in
-        log#f 4
+        log#info
           "Libfaad recognizes %S as AAC (%dHz,%d channels)."
           filename rate channels ;
           { Frame.
@@ -289,7 +287,7 @@ let get_type filename =
         let mp4 = Faad.Mp4.openfile_fd fd in
         let track = Faad.Mp4.find_aac_track mp4 in
         let rate, channels = Faad.Mp4.init mp4 dec track in
-           log#f 4
+           log#info
              "Libfaad recognizes %S as MP4 (%dHz,%d channels)."
              filename rate channels ;
            { Frame.
@@ -298,12 +296,12 @@ let get_type filename =
              midi  = 0 })
 
 let mp4_mime_types =
-  Conf.list ~p:(Decoder.conf_mime_types#plug "mp4")
+  Dtools.Conf.list ~p:(Decoder.conf_mime_types#plug "mp4")
     "Mime-types used for guessing MP4 format"
     ~d:["audio/mp4"; "application/mp4"]
 
 let mp4_file_extensions =
-  Conf.list ~p:(Decoder.conf_file_extensions#plug "mp4")
+  Dtools.Conf.list ~p:(Decoder.conf_file_extensions#plug "mp4")
     "File extensions used for guessing MP4 format"
     ~d:["m4a"; "m4b"; "m4p"; "m4v";
         "m4r"; "3gp"; "mp4"]
@@ -342,7 +340,7 @@ let () =
     else
     None)
 
-let log = Dtools.Log.make ["metadata";"mp4"]
+let log = Log.make ["metadata";"mp4"]
 
 let get_tags file =
   if not (Decoder.test_file ~mimes:mp4_mime_types#get
