@@ -23,7 +23,7 @@
 module Img = Image.RGBA32
 module P = Image.Generic.Pixel
 
-let log = Dtools.Log.make ["decoder";"camlimages"]
+let log = Log.make ["decoder";"camlimages"]
 
 (* TODO: find something more efficient? *)
 let load_image filename =
