@@ -54,7 +54,7 @@ module type Fdkaac_t =
         type param =
             [ `Afterburner of bool
             | `Aot of aot
-            | `Bandwidth of bool
+            | `Bandwidth of int
             | `Bitrate of int
             | `Bitrate_mode of bitrate_mode
             | `Granule_length of int
@@ -77,9 +77,14 @@ struct
     let encoder =
       Fdkaac.Encoder.create params.Fdkaac_format.channels
     in
+    let bandwidth = match params.Fdkaac_format.bandwidth with
+      | `Auto -> 0
+      | `Fixed b -> b
+    in
     let params = [
       `Aot params.Fdkaac_format.aot;
-      `Samplerate params.Fdkaac_format.samplerate;
+      `Bandwidth bandwidth;
+      `Samplerate (Lazy.force params.Fdkaac_format.samplerate);
       `Transmux params.Fdkaac_format.transmux;
       `Afterburner params.Fdkaac_format.afterburner;
     ] @ (
@@ -97,7 +102,7 @@ struct
   let encoder aac =
     let enc = create_encoder aac in
     let channels = aac.Fdkaac_format.channels in
-    let samplerate = aac.Fdkaac_format.samplerate in
+    let samplerate = (Lazy.force aac.Fdkaac_format.samplerate) in
     let samplerate_converter =
       Audio_converter.Samplerate.create channels
     in
