@@ -27,7 +27,7 @@ open Gstreamer_format
 module GU = Gstreamer_utils
 module Img = Image.RGBA32
 
-let log = Dtools.Log.make ["encoder"; "gstreamer"]
+let log = Log.make ["encoder"; "gstreamer"]
 
 type gst =
   {
