@@ -40,6 +40,12 @@ type aot =
      | `Mpeg_2 of mpeg2_aac
   ]
 
+type bandwidth =
+  [
+    | `Auto
+    | `Fixed of int
+  ]
+
 type bitrate_mode =
   [
      | `Constant
@@ -59,10 +65,11 @@ type transmux =
 type t = {
   afterburner    : bool;
   aot            : aot;
+  bandwidth      : bandwidth;
   bitrate_mode   : bitrate_mode;
   bitrate        : int;
   channels       : int;
-  samplerate     : int;
+  samplerate     : int Lazy.t;
   sbr_mode       : bool;
   transmux       : transmux
 }
@@ -114,4 +121,6 @@ let to_string m =
   Printf.sprintf "%%fdkaac(afterburner=%b,aot=%S,%s,channels=%d,\
                            samplerate=%d,sbr_mode=%b,transmux=%S)"
     m.afterburner (string_of_aot m.aot) br_info m.channels
-    m.samplerate m.sbr_mode (string_of_transmux m.transmux)
+    (Lazy.force m.samplerate) m.sbr_mode (string_of_transmux m.transmux)
+
+let bitrate m = m.bitrate * 1000
