@@ -26,7 +26,7 @@ open Gstreamer
 module GU = Gstreamer_utils
 module Img = Image.RGBA32
 
-let log = Dtools.Log.make ["io";"gstreamer"]
+let log = Log.make ["io";"gstreamer"]
 let gst_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "gstreamer")
 
 let string_of_state_change = function
@@ -57,7 +57,7 @@ object (self)
   val mutable element_m = Mutex.create ()
   val mutable element = None
 
-  method virtual log : Dtools.Log.t
+  method virtual log : Log.t
 
   method virtual make_element : ('a, 'b) element
 
@@ -83,11 +83,13 @@ object (self)
     in
     if should_run then
       try
-        self#log#f 3 "Restarting pipeline";
+        self#log#important "Restarting pipeline";
         Tutils.mutexify element_m (fun () ->
           begin match element with
             | None -> ()
-            | Some el -> ignore (Element.set_state el.bin Element.State_null)
+            | Some el ->
+               ignore (Element.set_state el.bin Element.State_null);
+               ignore (Element.get_state el.bin)
           end;
           let el = self#make_element in
           element <- Some el;
@@ -98,15 +100,15 @@ object (self)
         Tutils.mutexify restart_m (fun () ->
           restarting <- false) ();
         if retry_in >= 0. then
-          self#log#f 4 "An error occured while restarting pipeline, will retry in %.02f" retry_in
+          self#log#info "An error occured while restarting pipeline, will retry in %.02f" retry_in
         else
-          self#log#f 4 "Done restarting pipeline";
+          self#log#info "Done restarting pipeline";
         retry_in
       with exn ->
+        self#log#important "Error while restarting pipeline: %s" (Printexc.to_string exn);
+        self#log#info "Backtrace: %s" (Printexc.get_backtrace ());
         retry_in <- on_error exn;
-        self#log#f 3 "Error while restarting pipeline: %s" (Printexc.to_string exn);
-        self#log#f 4 "Backtrace: %s" (Printexc.get_backtrace ());
-        self#log#f 3 "Will retry again in %.02f" retry_in;
+        self#log#important "Will retry again in %.02f" retry_in;
         Tutils.mutexify restart_m (fun () ->
           restarting <- false) ();
         retry_in
@@ -180,6 +182,10 @@ object (self)
   method output_start =
     let el = self#get_element in
     ignore (Element.set_state el.bin Element.State_playing);
+    (* Don't uncomment the following line, it locks the program. I guess that
+       GStreamer is waiting for some data before answering that we are
+       playing. *)
+    (* ignore (Element.get_state el.bin); *)
     self#register_task ~priority:Tutils.Blocking Tutils.scheduler;
     if clock_safe then (gst_clock ())#register_blocking_source
 
@@ -197,6 +203,7 @@ object (self)
               if has_video then
                 App_src.end_of_stream (Utils.get_some el.video);
               ignore (Element.set_state el.bin Element.State_null);
+              ignore (Element.get_state el.bin);
               GU.flush ~log:self#log el.bin) ()
     in
     todo ();
@@ -221,7 +228,7 @@ object (self)
       else
         pipeline
     in
-    self#log#f 5 "GStreamer pipeline: %s" pipeline;
+    self#log#debug "GStreamer pipeline: %s" pipeline;
     let bin = Pipeline.parse_launch pipeline in
     let audio_src =
       if has_audio then
@@ -268,8 +275,8 @@ object (self)
         presentation_time <- Int64.add presentation_time duration;
         GU.flush ~log:self#log ~on_error:(fun err -> raise (Flushing_error err)) el.bin
       with e ->
-        self#log#f 3 "Error while processing output data: %s" (Printexc.to_string e);
-        self#log#f 4 "Stacktrace: %s" (Printexc.get_backtrace ());
+        self#log#important "Error while processing output data: %s" (Printexc.to_string e);
+        self#log#info "Stacktrace: %s" (Printexc.get_backtrace ());
         self#on_error e
 
   method output_reset = ()
@@ -442,11 +449,11 @@ object (self)
   inherit [string sink, Gstreamer.data sink] element_factory ~on_error
 
   initializer
-    rlog := (fun s -> self#log#f 3 "%s" s);
+    rlog := (fun s -> self#log#important "%s" s);
     let change_state s _ =
       try
         Printf.sprintf "Done. State change returned: %s"
-          (string_of_state_change (Element.set_state self#get_element.bin s) )
+          (string_of_state_change (Element.set_state self#get_element.bin s))
       with
         | e ->
             Printf.sprintf "Error while changing state: %s\n" (Printexc.to_string e)
@@ -462,8 +469,7 @@ object (self)
   method stype = Source.Fallible
   method remaining = -1
 
-  (* Source is ready when ready = true
-   * and gst has some audio or some video. *)
+  (* Source is ready when ready = true and gst has some audio or some video. *)
   val mutable ready = true
   method is_ready =
     let pending = function
@@ -477,7 +483,7 @@ object (self)
          (pending self#get_element.video))
     with
     | e ->
-      log#f 4 "Error when trying checking if ready: %s" (Printexc.to_string e);
+      log#info "Error when trying to check if ready: %s" (Printexc.to_string e);
       false
 
   method abort_track = ()
@@ -485,11 +491,13 @@ object (self)
   method wake_up activations =
     super#wake_up activations;
     try
-      ignore (Element.set_state self#get_element.bin Element.State_playing);
       self#register_task ~priority:Tutils.Blocking Tutils.scheduler;
+      ignore (Element.set_state self#get_element.bin Element.State_playing);
+      ignore (Element.get_state self#get_element.bin)
     with
-      | exn ->
-          self#on_error exn
+    | exn ->
+       self#log#info "Error setting state to playing: %s" (Printexc.to_string exn);
+       self#on_error exn
 
   method sleep =
     self#stop_task;
@@ -500,6 +508,7 @@ object (self)
               element <- None;
               (fun () ->
                 ignore (Element.set_state el.bin Element.State_null);
+                ignore (Element.get_state el.bin);
                 GU.flush ~log:self#log el.bin)
           | None -> fun () -> ()) ()
     in
@@ -511,26 +520,26 @@ object (self)
      if has_audio then
        Printf.sprintf
          "%s %s ! %s ! %s"
-         pipeline
-         (Utils.get_some audio_pipeline)
+         (pipeline ())
+         (Utils.get_some audio_pipeline ())
          (GU.Pipeline.decode_audio ())
          (GU.Pipeline.audio_sink ~channels "audio_sink")
      else
-       pipeline
+       pipeline ()
    in
    let pipeline =
      if has_video then
        Printf.sprintf
          "%s %s ! %s ! %s"
          pipeline
-         (Utils.get_some video_pipeline)
+         (Utils.get_some video_pipeline ())
          (GU.Pipeline.decode_video ())
          (GU.Pipeline.video_sink "video_sink")
      else
        pipeline
    in
-   log#f 5 "GStreamer pipeline: %s" pipeline;
-   let bin =  Pipeline.parse_launch pipeline in
+   log#debug "GStreamer pipeline: %s" pipeline;
+   let bin = Pipeline.parse_launch pipeline in
    let wrap_sink sink pull =
      let m = Mutex.create () in
      let counter = ref 0 in
@@ -604,12 +613,16 @@ object (self)
       GU.flush ~log:self#log ~on_error:(fun err -> raise (Flushing_error err)) el.bin
     with
       | Gstreamer.End_of_stream ->
+          self#log#info "End of stream.";
           ready <- false;
           if restart then
-            self#restart
+            (
+              self#log#info "Restarting.";
+              self#restart
+            )
       | exn ->
-          self#log#f 3 "Error while processing input data: %s" (Printexc.to_string exn);
-          self#log#f 4 "Stacktrace: %s" (Printexc.get_backtrace ());
+          self#log#important "Error while processing input data: %s" (Printexc.to_string exn);
+          self#log#info "Stacktrace: %s" (Printexc.get_backtrace ());
           self#on_error exn
 end
 
@@ -623,7 +636,7 @@ let input_proto =
           If returned value is positive, connection will be tried again after \
           this amount of time (in seconds)." ;
     "restart", Lang.bool_t, Some (Lang.bool true),
-    Some "Restart input on end of stream event";
+    Some "Restart input on end of stream event.";
     "max", Lang.float_t, Some (Lang.float 10.),
     Some "Maximum duration of the buffered data." ;
   ]
@@ -640,11 +653,11 @@ let () =
   in
   let proto = input_proto @ 
     [
-      "pipeline", Lang.string_t, Some (Lang.string ""),
+      "pipeline", Lang.string_getter_t 1, Some (Lang.string ""),
       Some "Main GStreamer pipeline.";
-      "audio_pipeline", Lang.string_t, Some (Lang.string "audiotestsrc"),
+      "audio_pipeline", Lang.string_getter_t 2, Some (Lang.string "audiotestsrc"),
       Some "Audio pipeline to input from.";
-      "video_pipeline", Lang.string_t, Some (Lang.string "videotestsrc"),
+      "video_pipeline", Lang.string_getter_t 3, Some (Lang.string "videotestsrc"),
       Some "Video pipeline to input from.";
     ]
   in
@@ -654,16 +667,16 @@ let () =
     ~flags:[]
     ~descr:"Stream audio+video from a GStreamer pipeline."
     (fun p kind ->
-      let pipeline = Lang.to_string (List.assoc "pipeline" p) in
-      let audio_pipeline = Lang.to_string (List.assoc "audio_pipeline" p) in
-      let video_pipeline = Lang.to_string (List.assoc "video_pipeline" p) in
+      let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
+      let audio_pipeline = Lang.to_string_getter (List.assoc "audio_pipeline" p) in
+      let video_pipeline = Lang.to_string_getter (List.assoc "video_pipeline" p) in
       ((new audio_video_input p kind (pipeline,Some audio_pipeline,Some video_pipeline)):>Source.source))
 
 let () =
   let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
   let proto = input_proto @ 
     [
-      "pipeline", Lang.string_t, Some (Lang.string "audiotestsrc"),
+      "pipeline", Lang.string_getter_t 1, Some (Lang.string "audiotestsrc"),
       Some "GStreamer pipeline to input from.";
     ]
   in
@@ -673,14 +686,14 @@ let () =
     ~flags:[]
     ~descr:"Stream audio from a GStreamer pipeline."
     (fun p kind ->
-      let pipeline = Lang.to_string (List.assoc "pipeline" p) in
-      ((new audio_video_input p kind ("",Some pipeline,None)):>Source.source))
+      let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
+      ((new audio_video_input p kind ((fun()->""),Some pipeline,None)):>Source.source))
 
 let () =
   let k = Lang.kind_type_of_kind_format ~fresh:1 (Lang.video_n 1) in
   let proto = input_proto @ 
     [
-      "pipeline", Lang.string_t, Some (Lang.string "videotestsrc"),
+      "pipeline", Lang.string_getter_t 1, Some (Lang.string "videotestsrc"),
       Some "GStreamer pipeline to input from.";
     ]
   in
@@ -690,5 +703,5 @@ let () =
     ~flags:[]
     ~descr:"Stream video from a GStreamer pipeline."
     (fun p kind ->
-      let pipeline = Lang.to_string (List.assoc "pipeline" p) in
-      ((new audio_video_input p kind ("",None,Some pipeline)):>Source.source))
+      let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
+      ((new audio_video_input p kind ((fun()->""),None,Some pipeline)):>Source.source))
