@@ -25,11 +25,13 @@ open Lang_encoders
 
 let make params =
   let defaults = { Wav_format.
-                    channels   = 2 ;
                     samplesize = 16;
                     header = true;
                     duration = None;
-                    samplerate = 44100 } in
+                    (* We use a hardcoded value in order not to force the evaluation of the
+                       number of channels too early, see #933. *)
+                    channels = 2;
+                    samplerate = Frame.audio_rate } in
   let wav =
     List.fold_left
       (fun f ->
@@ -47,7 +49,7 @@ let make params =
           | ("duration",{ term = Float d; _ }) ->
               { f with Wav_format.duration = Some d }
           | ("samplerate",{ term = Int i; _ }) ->
-              { f with Wav_format.samplerate = i }
+              { f with Wav_format.samplerate = Lazy.from_val i }
           | ("samplesize",({ term = Int i; _ } as t)) ->
               if i <> 8 && i <> 16 && i <> 24 && i <> 32 then
                 raise (Error (t,"invalid sample size")) ;
