@@ -65,6 +65,11 @@ object (self)
 
   method is_ready = List.exists (fun (_,s) -> s#is_ready) sources
 
+  method seek n =
+    match sources with
+      | (_,s) :: [] -> s#seek n
+      | _ -> 0
+
   (* We fill the buffer as much as possible, removing internal breaks.
    * Every ready source is asked for as much data as possible, by asking
    * it to fill the intermediate [tmp] buffer. Then that data is added
@@ -128,7 +133,7 @@ object (self)
                while Frame.is_partial buffer && s#is_ready do
                  incr get_count ;
                  if !get_count > Lazy.force Frame.size then
-                   self#log#f 2
+                   self#log#severe
                      "Warning: there may be an infinite sequence of empty tracks!" ;
                  get_again s buffer
                done
@@ -213,13 +218,13 @@ let () =
        let renorm = Lang.to_bool (List.assoc "normalize" p) in
          if List.length weights <> List.length sources then
            raise
-             (Lang.Invalid_value
+             (Lang_errors.Invalid_value
                 ((List.assoc "weights" p),
                  "there should be as many weights as sources")) ;
          new add ~kind ~renorm
                (List.map2 (fun w s -> (w,s)) weights sources)
                (fun _ -> ())
-               (fun _ buf tmp -> Img.add buf tmp))
+               (fun _ buf tmp -> Img.add tmp buf))
 
 let tile_pos n =
   let vert l x y x' y' =
@@ -284,7 +289,7 @@ let () =
        let video_init buf = video_loop 0 buf buf in
          if List.length weights <> List.length sources then
            raise
-             (Lang.Invalid_value
+             (Lang_errors.Invalid_value
                 ((List.assoc "weights" p),
                  "there should be as many weights as sources")) ;
          new add ~kind ~renorm
