@@ -53,6 +53,8 @@ object (self)
 
   method abort_track = source#abort_track
 
+  method seek = source#seek
+
   val mutable notes_on = []
 
   method private get_frame buf =
@@ -80,7 +82,7 @@ object (self)
                   ans := (t,n,m) :: !ans
                 with
                   | Not_found ->
-                    self#log#f 3 "Could not parse chord '%s'." c
+                    self#log#important "Could not parse chord '%s'." c
               ) (Hashtbl.find_all m metadata_name)
           ) meta;
         List.rev !ans
@@ -115,7 +117,7 @@ object (self)
                  | "dim" ->
                      play t [c;c+3;c+6]
                  | m ->
-                     self#log#f 5 "Unknown mode: %s\n%!" m
+                     self#log#debug "Unknown mode: %s\n%!" m
              );
         ) chords
 end
