@@ -28,6 +28,7 @@ object
 
   method stype = source#stype
   method remaining = source#remaining
+  method seek = source#seek
   method is_ready = source#is_ready
   method abort_track = source#abort_track
 
@@ -45,7 +46,7 @@ let () =
   [ "", Lang.source_t k, None, None ]
     ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
-    ~descr:"Clip sound."
+    ~descr:"Clip samples, i.e. ensure that all values are between -1 and 1: values lower than -1 become -1 and values higher than 1 become 1."
     (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
