@@ -29,6 +29,7 @@ object
 
   method stype = source#stype
   method remaining = source#remaining
+  method seek = source#seek
   method is_ready = source#is_ready
   method abort_track = source#abort_track
 
@@ -80,7 +81,7 @@ let () =
        let feedback =
          (* Check the initial value, wrap the getter with a converter. *)
          if feedback () > 0. then
-           raise (Lang.Invalid_value (f "feedback",
+           raise (Lang_errors.Invalid_value (f "feedback",
                                       "feedback should be negative"));
          fun () -> Audio.lin_of_dB (feedback ())
        in
