@@ -32,6 +32,7 @@ object
 
   method stype = source#stype
   method remaining = source#remaining
+  method seek = source#seek
   method is_ready = source#is_ready
   method abort_track = source#abort_track
 
@@ -109,7 +110,7 @@ let () =
           | "high" -> High_pass
           | "band" -> Band_pass
           | "notch" -> Notch
-          | _ -> raise (Lang.Invalid_value
+          | _ -> raise (Lang_errors.Invalid_value
                           (mode,
                            "valid values are low|high|band|notch"))
       in
