@@ -21,7 +21,6 @@
  *****************************************************************************)
 
 open Source
-open Dtools
 open Extralib
 
 module Img = Image.RGBA32
@@ -52,6 +51,7 @@ object
 
   method stype = source#stype
   method remaining = source#remaining
+  method seek = source#seek
   method is_ready = source#is_ready
   method abort_track = source#abort_track
 
@@ -372,7 +372,7 @@ let register_plugins () =
       | End_of_file -> Unix.closedir dir
     with
     | Unix.Unix_error (e,_,_) ->
-      log#f 4 "Error while loading directory %s: %s"
+      log#info "Error while loading directory %s: %s"
         plugins_dir (Unix.error_message e)
   in
   List.iter add plugin_dirs
