@@ -22,26 +22,20 @@
 
 open Source
 open Ladspa
-open Dtools
 
 type t = Float | Int | Bool
 
 let log = Log.make ["LADSPA extension"]
 
-let ladspa_enable =
-  try
-    let venv = Unix.getenv "LIQ_LADSPA" in
-      venv = "1" || venv = "true"
-  with
-    | Not_found -> true
+let conf_ladspa =
+  Dtools.Conf.void ~p:(Utils.conf#plug "ladspa") "Lasdpa Configuration"
 
-let plugin_dirs =
-  try
-    let path = Unix.getenv "LIQ_LADSPA_PATH" in
-      Pcre.split ~pat:":" path
-  with
-    | Not_found -> ["/usr/lib64/ladspa";"/usr/lib/ladspa";"/usr/local/lib/ladspa"]
+let conf_enable =
+  Dtools.Conf.bool ~p:(conf_ladspa#plug "enable") ~d:true "Enable Ladspa "
 
+let conf_dirs =
+  Dtools.Conf.list ~p:(conf_ladspa#plug "dirs") ~d:["/usr/lib64/ladspa";"/usr/lib/ladspa";"/usr/local/lib/ladspa"]
+    "Directories to search for plugins"
 
 let port_t d p =
   if Descriptor.port_is_boolean d p then Bool
@@ -56,6 +50,8 @@ object
 
   method remaining = source#remaining
 
+  method seek = source#seek
+
   method is_ready = source#is_ready
 
   method abort_track = source#abort_track
@@ -268,7 +264,7 @@ let params_of_descr d =
                        | None -> ()
                      end ;
                      bounds :=
-                     !bounds ^ "<code>" ^ (Utils.normalize_parameter_string (Descriptor.port_name d p)) ^ "</code>";
+                     !bounds ^ "`" ^ (Utils.normalize_parameter_string (Descriptor.port_name d p)) ^ "`";
                      begin match max with
                        | Some f ->
                            begin match t with
@@ -419,11 +415,12 @@ let register_plugins () =
           | End_of_file -> Unix.closedir dir
     with
       | Unix.Unix_error (e,_,_) ->
-          log#f 4 "Error while loading directory %s: %s"
+          log#info "Error while loading directory %s: %s"
             plugins_dir (Unix.error_message e)
   in
-    List.iter add plugin_dirs
+    List.iter add conf_dirs#get
 
 let () =
-  if ladspa_enable then
-    register_plugins ()
+  Configure.at_init (fun () ->
+    if conf_enable#get then
+      register_plugins ())
