@@ -40,38 +40,8 @@ type chunk = {
   mutable len: int
 }
 
-exception Not_enough_data
-
-let read_header =
-  let really_input buf s ofs len =
-    if Buffer.length buf < len then
-      raise Not_enough_data;
-    Buffer.blit buf 0 s ofs len;
-    Utils.buffer_drop buf len
-  in
-  let b = Bytes.create 1 in
-  let input_byte buf =
-    if Buffer.length buf < 1 then
-      raise Not_enough_data;
-    Buffer.blit buf 0 b 0 1;
-    Utils.buffer_drop buf 1;
-    Char.code (Bytes.get b 0)
-  in
-  let input buf s ofs len =
-    let len =
-      max (Buffer.length buf) len
-    in
-    Buffer.blit buf 0 s ofs len;
-    Utils.buffer_drop buf len;
-    len
-  in
-  let seek _ _ = assert false in
-  let close _ = () in
-  Wav_aiff.read_header {Wav_aiff.
-    really_input;input_byte;input;seek;close
-  }
-    
-class pipe ~kind ~process ~bufferize ~max ~restart ~restart_on_error (source:source) =
+class pipe ~kind ~replay_delay ~data_len ~process ~bufferize ~max ~restart
+           ~restart_on_error (source:source) =
   (* We need a temporary log until the source has an id *)
   let log_ref = ref (fun _ -> ()) in
   let log = (fun x -> !log_ref x) in
@@ -80,71 +50,86 @@ class pipe ~kind ~process ~bufferize ~max ~restart ~restart_on_error (source:sou
   let audio_src_rate = float sample_rate in
   let channels = (Frame.type_of_kind kind).Frame.audio in
   let abg_max_len = Frame.audio_of_seconds max in
+  let replay_delay =
+    Frame.audio_of_seconds replay_delay
+  in
   let samplesize = ref 16 in
   let converter = ref
     (Rutils.create_from_iff ~format:`Wav ~channels ~samplesize:!samplesize
                             ~audio_src_rate)
   in
+  let len =
+    match data_len with
+      | x when x < 0 -> None
+      | l -> Some l
+  in
   let header = 
     Bytes.unsafe_of_string
       (Wav_aiff.wav_header ~channels ~sample_rate
-                           ~sample_size:16 ())
+                           ?len ~sample_size:16 ())
   in
   let on_start push =
     Process_handler.write header push;
     `Continue
   in
   let abg = Generator.create ~log ~kind `Audio in
-  let buf = Buffer.create 1024 in
   let mutex = Mutex.create () in
+  let replay_pending = ref [] in
   let next_stop = ref `Nothing in
-  let is_first = ref true in
-  let process_data () =
-    (* Round to a multiple of sample_size * channels *)
-    let len =
-      let ratio =
-        !samplesize * channels
-      in
-      (Buffer.length buf / ratio) * ratio
-    in
-    let data =
-      Buffer.sub buf 0 len
-    in
-    Utils.buffer_drop buf len;
-    let data = !converter data in
-    let len = Array.length data.(0) in
-    let buffered = Generator.length abg in
-    Generator.put_audio abg data 0 (Array.length data.(0));
-    if abg_max_len < buffered+len then
-      `Delay (Frame.seconds_of_audio (buffered+len-abg_max_len))
-    else
-      `Continue
-  in
+  let header_read = ref false in
   let on_stdout pull =
-    Buffer.add_bytes buf
-      (Process_handler.read 1024 pull);
-    let done_with_header =
+    if not (!header_read) then
+     begin
+      let wav = Wav_aiff.read_header Wav_aiff.callback_ops pull in
+      header_read := true; 
       Tutils.mutexify mutex (fun () ->
-        if !is_first then
-          let contents = Buffer.contents buf in
-          try
-            let wav = read_header buf in
-            if Wav_aiff.channels wav <> channels then
-              failwith "Invalid channels from pipe process!";
-            samplesize := Wav_aiff.sample_size wav;
-            converter :=
-              Rutils.create_from_iff ~format:`Wav ~channels
-                ~samplesize:!samplesize
-                ~audio_src_rate:(float (Wav_aiff.sample_rate wav));
-            is_first := false;
-            true
-           with Not_enough_data ->
-             Buffer.reset buf;
-             Buffer.add_string buf contents;
-             false
-        else true) ()
-    in
-    if done_with_header then process_data () else `Continue
+        if Wav_aiff.channels wav <> channels then
+          failwith "Invalid channels from pipe process!";
+        samplesize := Wav_aiff.sample_size wav;
+        converter :=
+          Rutils.create_from_iff ~format:`Wav ~channels
+            ~samplesize:!samplesize
+            ~audio_src_rate:(float (Wav_aiff.sample_rate wav))) ();
+        `Reschedule Tutils.Non_blocking
+     end
+    else
+      let data = Process_handler.read 1024 pull in
+      let data = !converter (Bytes.unsafe_to_string data) in
+      let len = Array.length data.(0) in
+      let buffered = Generator.length abg in
+      Generator.put_audio abg data 0 (Array.length data.(0));
+
+      let to_replay = Tutils.mutexify mutex (fun () ->
+        let pending = !replay_pending in
+        let to_replay, pending = List.fold_left (fun ((pos,b),cur) (pos',b') -> 
+          if pos'+len > replay_delay then
+           begin
+            if pos > 0 then
+              log "Cannot replay multiple element at once.. Picking up the most recent";
+            if pos > 0 && pos < pos' then (pos,b),cur else (pos',b'), cur
+           end
+          else (pos,b),(pos'+len,b')::cur) ((-1,`Nothing),[]) pending
+        in 
+        replay_pending := pending;
+        to_replay) ()
+      in
+      begin
+       match to_replay with
+         | -1, _ -> ()
+         | _, `Break_and_metadata m ->
+             Generator.add_metadata abg m;
+             Generator.add_break abg
+         | _, `Metadata m ->
+             Generator.add_metadata abg m
+         | _, `Break ->
+             Generator.add_break abg
+         | _ -> ()
+      end; 
+
+      if abg_max_len < buffered+len then
+        `Delay (Frame.seconds_of_audio (buffered+len-abg_max_len))
+      else
+        `Continue
   in
   let on_stderr stderr =
     (!log_error) (Bytes.unsafe_to_string (Process_handler.read 1024 stderr));
@@ -153,9 +138,7 @@ class pipe ~kind ~process ~bufferize ~max ~restart ~restart_on_error (source:sou
   let on_stop = Tutils.mutexify mutex (fun e ->
     let ret = !next_stop in
     next_stop := `Nothing;
-    is_first := true;
-    ignore(process_data ());
-    Buffer.reset buf;
+    header_read := false;
     let should_restart =
       match e with
         | `Status s when s <> (Unix.WEXITED 0) ->
@@ -240,9 +223,21 @@ object(self)
       let wlen = min 1024 len in
       let ret = pusher sbuf ofs wlen in
       if ret = len then begin
-        Tutils.mutexify mutex (fun () -> next_stop := next) ();
+        let action =
+          if next <> `Nothing && replay_delay >= 0 then
+           begin
+            Tutils.mutexify mutex (fun () ->
+              replay_pending := (0,next)::!replay_pending) ();
+            `Continue
+           end
+          else
+           begin
+            Tutils.mutexify mutex (fun () -> next_stop := next) ();
+            if next <> `Nothing then `Stop else `Continue
+           end
+        in
         ignore(Queue.take to_write); 
-        if next <> `Nothing then `Stop else `Continue
+        action
       end else begin
         chunk.ofs <- ofs+ret;
         chunk.len <- len-ret;
@@ -266,10 +261,10 @@ object(self)
   method wake_up _ =
     source#get_ready [(self:>source)];
     (* Now we can create the log function *)
-    log_ref := self#log#f 4 "%s";
-    log_error := self#log#f 5 "%s";
+    log_ref := self#log#info "%s";
+    log_error := self#log#debug "%s";
     handler <- Some (Process_handler.run ~on_stop ~on_start ~on_stdout 
-                                         ~on_stdin:self#on_stdin
+                                         ~on_stdin:self#on_stdin ~priority:Tutils.Blocking
                                          ~on_stderr ~log process)
 
   method abort_track = source#abort_track
@@ -278,6 +273,7 @@ object(self)
     Tutils.mutexify mutex (fun () ->
       try
         next_stop := `Sleep;
+        replay_pending := [];
         Process_handler.stop self#get_handler;
         handler <- None
       with Process_handler.Finished -> ()) ()
@@ -290,6 +286,21 @@ let proto =
     "process", Lang.string_t, None,
     Some "Process used to pipe data to.";
 
+    "replay_delay", Lang.float_t, Some (Lang.float (-1.)),
+    Some "Replay track marks and metadata from the input source \
+          on the output after a given delay. If negative (default) \
+          close and flush the process on each track and metadata to \
+          get an exact timing.";
+
+    "data_length", Lang.int_t, Some (Lang.int (-1)),
+    Some "Length passed in the WAV data chunk. \
+          Data is streamed so no the consuming program \
+          should process it as it comes. Some program \
+          operate better when this value is set to `0`, \
+          some other when it is set to the maximum length \
+          allowed by the WAV specs. Use any negative value \
+          to set to maximum length.";
+
     "buffer", Lang.float_t, Some (Lang.float 1.),
     Some "Duration of the pre-buffered data." ;
 
@@ -307,15 +318,17 @@ let proto =
 
 let pipe p kind =
   let f v = List.assoc v p in
-  let process, bufferize, max, restart, restart_on_error, src =
+  let process, replay_delay, data_len, bufferize, max, restart, restart_on_error, src =
     Lang.to_string (f "process"),
+    Lang.to_float (f "replay_delay"),
+    Lang.to_int (f "data_length"),
     Lang.to_float (f "buffer"),
     Lang.to_float (f "max"),
     Lang.to_bool (f "restart"),
     Lang.to_bool (f "restart_on_error"),
     Lang.to_source (f "")
   in
-  ((new pipe ~kind ~bufferize ~max ~restart ~restart_on_error ~process src):>source)
+  ((new pipe ~kind ~replay_delay ~data_len ~bufferize ~max ~restart ~restart_on_error ~process src):>source)
 
 let () =
   Lang.add_operator "pipe" proto
