@@ -44,11 +44,13 @@ type child = {
   * or only at track limits (sensitive). *)
 type track_mode = Sensitive | Insensitive
 
-class virtual switch ~kind ~name
+class virtual switch ~kind ~name ~override_meta ~transition_length
   ?(mode=(fun () -> true)) ?(replay_meta=true) (cases : child list) =
 object (self)
   inherit operator ~name kind (List.map (fun x -> x.source) cases)
 
+  val mutable transition_length = transition_length
+
   val mutable selected : (child*source) option = None
 
   (** We have to explictly manage our children as they are dynamically created
@@ -132,14 +134,14 @@ object (self)
       | Some c ->
           begin match selected with
             | None ->
-                self#log#f 3 "Switch to %s." c.source#id ;
+                self#log#important "Switch to %s." c.source#id ;
                 (* The source is already ready, this call is only there for
                  * allowing an uniform treatment of switches, triggering
                  * a #leave call. *)
                 c.source#get_ready activation ;
                 selected <- Some (c,c.source)
             | Some (old_c,old_s) when old_c != c ->
-                self#log#f 3 "Switch to %s with%s transition."
+                self#log#important "Switch to %s with%s transition."
                   c.source#id
                   (if forget then " forgetful" else "") ;
                 old_s#leave (self:>source) ;
@@ -168,6 +170,12 @@ object (self)
                          c.transition
                          [ "",Lang.source old_source ;
                            "",Lang.source new_source ])
+                  in
+                  let s =
+                    new Max_duration.max_duration ~kind ~override_meta ~duration:transition_length s
+                  in
+                  let s =
+                    new Sequence.sequence ~kind ~merge:true [s;new_source]
                   in
                     Clock.unify s#clock self#clock ;
                     s#get_ready activation ;
@@ -235,6 +243,10 @@ object (self)
       | Some (_,s) -> s#abort_track
       | None -> ()
 
+  method seek n =
+    match selected with
+      | Some (_,s) -> s#seek n
+      | None -> 0
 end
 
 (** Common tools for Lang bindings of switch operators *)
@@ -243,6 +255,13 @@ let common kind = [
   "track_sensitive", Lang.bool_getter_t 0, Some (Lang.bool true),
   Some "Re-select only on end of tracks." ;
 
+  "transition_length", Lang.float_t, Some (Lang.float 5.),
+  Some "Maximun transition duration.";
+
+  "override", Lang.string_t, Some (Lang.string "liq_transition_length"),
+  Some "Metadata field which, if present and containing a float, \
+        overrides the `transition_length` parameter.";
+
   "replay_metadata", Lang.bool_t, Some (Lang.bool true),
   Some "Replay the last metadata of a child when switching to it \
         in the middle of a track." ;
@@ -257,7 +276,7 @@ let common kind = [
   Lang.list_t transition_t,
   Some (Lang.list ~t:transition_t []),
   Some "Transition functions, \
-        padded with <code>fun (x,y) -> y</code> functions."
+        padded with `fun (x,y) -> y` functions."
 ]
 
 let default_transition k =
@@ -271,7 +290,7 @@ let extract_common ~kind p l =
   let tr =
     let tr = Lang.to_list (List.assoc "transitions" p) in
     let ltr = List.length tr in
-      if ltr > l then raise (Lang.Invalid_value
+      if ltr > l then raise (Lang_errors.Invalid_value
                                ((List.assoc "transitions" p),
                                 "Too many transitions")) ;
       if ltr < l then
@@ -280,7 +299,14 @@ let extract_common ~kind p l =
         tr
   in
   let replay = Lang.to_bool (List.assoc "replay_metadata" p) in
-    replay,ts,tr
+  let tl =
+    Frame.master_of_seconds
+      (Lang.to_float (List.assoc "transition_length" p))
+  in
+  let override_meta =
+    Lang.to_string (List.assoc "override" p)
+  in
+    replay,ts,tr,tl,override_meta
 
 (** Switch: switch according to user-defined predicates. *)
 
@@ -292,11 +318,11 @@ let trivially_true = function
 
 let third (_,_,s) = s
 
-class lang_switch ~kind
+class lang_switch ~kind ~override_meta ~transition_length
   mode ?replay_meta (children : (Lang.value * bool * child) list) =
 object
   inherit
-    switch ~name:"switch" ~kind ~mode ?replay_meta (List.map third children)
+    switch ~name:"switch" ~kind ~mode ~override_meta ~transition_length ?replay_meta (List.map third children)
 
   method private select =
     let selected s =
@@ -333,7 +359,7 @@ let () =
   let proto =
     [ "single", Lang.list_t Lang.bool_t, Some (Lang.list ~t:Lang.bool_t []),
       Some "Forbid the selection of a branch for two tracks in a row. \
-            The empty list stands for <code>[false,...,false]</code>." ;
+            The empty list stands for `[false,...,false]`." ;
       "", Lang.list_t (Lang.product_t pred_t (Lang.source_t kind)), None,
       Some "Sources with the predicate telling when they can be played." ]
   in
@@ -351,7 +377,7 @@ let () =
                   pred, Lang.to_source s)
            (Lang.to_list (List.assoc "" p))
          in
-         let replay_meta,ts,tr =
+         let replay_meta,ts,tr,tl,override_meta =
            extract_common ~kind p (List.length children)
          in
          let singles =
@@ -372,17 +398,17 @@ let () =
          let children =
            try List.map2 (fun (d,s) single -> d,single,s) children singles with
              | Invalid_argument s when s = "List.map2" ->
-                 raise (Lang.Invalid_value
+                 raise (Lang_errors.Invalid_value
                           (List.assoc "single" p,
                            "there should be exactly one flag per children"))
          in
-           new lang_switch ~kind ~replay_meta ts children)
+           new lang_switch ~kind ~replay_meta ~override_meta ~transition_length:tl ts children)
 
 (** Fallback selector: switch to the first ready source. *)
-class fallback ~kind ?replay_meta mode children =
+class fallback ~kind ~override_meta ~transition_length ?replay_meta mode children =
 object
 
-  inherit switch ~name:"fallback" ~kind ~mode ?replay_meta children
+  inherit switch ~name:"fallback" ~kind ~mode ~override_meta ~transition_length ?replay_meta children
 
   method private select =
     try Some (List.find (fun s -> s.source#is_ready) children) with
@@ -410,7 +436,7 @@ let () =
       ~kind:(Lang.Unconstrained kind)
       (fun p kind ->
          let children = Lang.to_source_list (List.assoc "" p) in
-         let replay_meta,ts,tr =
+         let replay_meta,ts,tr,tl,override_meta =
            extract_common ~kind p (List.length children)
          in
          let children =
@@ -418,14 +444,14 @@ let () =
              (fun t s -> { transition = t ; cur_meta = None ; source = s })
              tr children
          in
-           new fallback ~kind ~replay_meta ts children)
+           new fallback ~kind ~replay_meta ~override_meta ~transition_length:tl ts children)
 
 (** Random switch *)
 exception Found of child
-class random ~kind ?replay_meta strict mode children =
+class random ~kind ~override_meta ~transition_length ?replay_meta strict mode children =
   let name = if strict then "rotate" else "random" in
 object
-  inherit switch ~name ~kind ?replay_meta ~mode (List.map snd children)
+  inherit switch ~name ~kind ~override_meta ~transition_length ?replay_meta ~mode (List.map snd children)
 
   val mutable pos = -1
 
@@ -435,7 +461,7 @@ object
   method private select =
     let (ready_list,n) =
       List.fold_left
-        (fun (l,k) (w,s) -> if s.source#is_ready then (s,w)::l,(k+w) else l,k)
+        (fun (l,k) (w,s) -> let w = w() in if s.source#is_ready then (s,w)::l,(k+w) else l,k)
         ([],0) children
     in
       if n = 0 then None else
@@ -461,27 +487,27 @@ let () =
   let add name strict descr weight_descr =
     let kind = Lang.univ_t 1 in
     Lang.add_operator name ~descr ~category:Lang.TrackProcessing
-      (common  kind @
-       [ "weights", Lang.list_t Lang.int_t, Some (Lang.list ~t:Lang.int_t []),
+      (common kind @
+       [ "weights", Lang.list_t (Lang.int_getter_t 2), Some (Lang.list ~t:(Lang.int_getter_t 2) []),
          Some weight_descr ;
          "", Lang.list_t (Lang.source_t kind), None, None ])
       ~kind:(Lang.Unconstrained kind)
       (fun p kind ->
          let children = Lang.to_source_list (List.assoc "" p) in
-         let replay_meta,ts,tr =
+         let replay_meta,ts,tr,tl,override_meta =
            extract_common ~kind p (List.length children)
          in
          let weights =
-           List.map Lang.to_int (Lang.to_list (List.assoc "weights" p))
+           List.map Lang.to_int_getter (Lang.to_list (List.assoc "weights" p))
          in
          let weights =
            if weights <> [] then weights else
-             Utils.make_list (List.length children) 1
+             Utils.make_list (List.length children) (fun () -> 1)
          in
          let children =
            if List.length weights <> List.length children then
              raise
-               (Lang.Invalid_value
+               (Lang_errors.Invalid_value
                   ((List.assoc "weights" p),
                    "there should be as many weights as sources")) ;
            List.map2
@@ -491,7 +517,7 @@ let () =
                 (fun tr s -> { transition = tr ; source = s ; cur_meta = None })
                 tr children)
          in
-           new random ~kind ~replay_meta strict ts children)
+           new random ~kind ~replay_meta ~override_meta ~transition_length:tl strict ts children)
   in
     add "random" false
       "At the beginning of every track, select a random ready child."
