@@ -22,8 +22,6 @@
 
 (** Output using SDL lib. *)
 
-open Sdl
-
 class output ~infallible ~on_start ~on_stop ~autostart ~kind source =
   let video_width    = Lazy.force Frame.video_width in
   let video_height   = Lazy.force Frame.video_height in
@@ -42,7 +40,7 @@ object (self)
     ignore (Sdlvideo.set_video_mode
               ~w:video_width ~h:video_height
               ~bpp:32 [`ANYFORMAT;`DOUBLEBUF]) ;
-    self#log#f 4 "Initialized SDL video surface with %dbpp."
+    self#log#info "Initialized SDL video surface with %dbpp."
       (Sdlvideo.surface_bpp (Sdlvideo.get_video_surface ()))
 
   (** We don't care about latency. *)
