@@ -20,7 +20,7 @@
 
  *****************************************************************************)
 
-let log = Dtools.Log.make ["playlist";"xml"]
+let log = Log.make ["playlist";"xml"]
 
 let conf_xml =
   Dtools.Conf.list ~p:(Playlist_parser.conf_mime_types#plug "xml")
@@ -44,7 +44,7 @@ let tracks ?pwd s =
       (fun (a,b) -> recode_metas a, Playlist_parser.get_file ?pwd b) 
       (Xmlplaylist.tracks s)
   with
-    | Xmlplaylist.Error(e) -> log#f 5 "Parsing failed: %s" 
+    | Xmlplaylist.Error(e) -> log#debug "Parsing failed: %s" 
                                  (Xmlplaylist.string_of_error e) ;
 		              raise (Xmlplaylist.Error(e))
 
