@@ -79,7 +79,7 @@ object (self)
     match device with
       | Some d -> d
       | None ->
-          self#log#f 3 "Using ALSA %s." (Alsa.get_version ()) ;
+          self#log#info "Using ALSA %s." (Alsa.get_version ()) ;
           let dev = Pcm.open_pcm alsa_device [Pcm.Capture] [] in
           let params = Pcm.get_params dev in
           begin try
@@ -88,7 +88,7 @@ object (self)
           with
             | _ ->
                 (* If we can't get floats we fallback on interleaved s16le *)
-                self#log#f 2 "Falling back on interleaved S16LE";
+                self#log#severe "Falling back on interleaved S16LE";
                 Pcm.set_access dev params Pcm.Access_rw_interleaved ;
                 Pcm.set_format dev params Pcm.Format_s16_le ;
                 read_fun <-
@@ -98,8 +98,12 @@ object (self)
                      Audio.S16LE.to_audio sbuf 0 buf ofs r;
                      r)
           end ;
-          sample_freq <-
-            Pcm.set_rate_near dev params sample_freq Dir_eq; (* TODO: resample *)
+          sample_freq <- Pcm.set_rate_near dev params sample_freq Dir_eq;
+          (* TODO: resample *)
+          if sample_freq <> Lazy.force Frame.audio_rate then
+            self#log#important "Got a sampling frequency of %d instead of %d \
+                                (TODO: should be resampled in the future)."
+              sample_freq (Lazy.force Frame.audio_rate);
           Pcm.set_channels dev params buffer_chans ;
           Pcm.set_params dev params ;
           Pcm.prepare dev ;
@@ -121,12 +125,12 @@ object (self)
         begin
          match e with
            | Buffer_xrun ->
-               self#log#f 2 "Overrun!"
-             | _ -> self#log#f 2 "Alsa error: %s" (string_of_error e)
+               self#log#important "Overrun!"
+           | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
         end ;
         if e = Buffer_xrun || e = Suspended || e = Interrupted then
          begin
-          self#log#f 2 "Trying to recover.." ;
+          self#log#severe "Trying to recover.." ;
           Pcm.recover dev e
          end
         else raise e
