@@ -22,7 +22,7 @@
 
 open Source
 
-let log = Dtools.Log.make ["input";"jack"]
+let log = Log.make ["input";"jack"]
 
 let bjack_clock =
   Tutils.lazy_cell (fun () -> new Clock.self_sync "bjack")
@@ -52,6 +52,7 @@ object (self)
 
   method private sleep =
     active_source#sleep ;
+    ioring#sleep ;
     if clock_safe then (bjack_clock ())#unregister_blocking_source
 
   method stype = Infallible
