@@ -48,7 +48,7 @@ object (self)
 
   val mutable cur_meta : Request.metadata option = None
 
-  method virtual private log : Dtools.Log.t
+  method virtual private log : Log.t
 
   method seek len =
     if not seek || len <= 0 then 0 else
@@ -61,12 +61,17 @@ object (self)
 
   method private length = Generator.length generator
 
+  val mutable last_buffering_warning = -1
+
   method is_ready =
     let r = self#length in
-      if buffering then begin
+    if buffering then begin
         (* We have some data, but not enough for safely starting to play it. *)
-        if bufferize > 0 && r <= bufferize then
-          self#log#f 6 "Not ready: need more buffering (%i/%i)." r bufferize ;
+        if bufferize > 0 && r <= bufferize && r <> last_buffering_warning then
+          (
+            last_buffering_warning <- r;
+            self#log#debug "Not ready: need more buffering (%i/%i)." r bufferize;
+          );
         r > bufferize
       end else begin
         (* This only happens if the end of track has not been played yet,
@@ -74,7 +79,7 @@ object (self)
          * that we're not accumulating data, but it means that we don't know
          * yet that we'll stop playing it until the buffer is full enough. *)
         if r = 0 then
-          self#log#f 4 "Not ready for a new track: empty buffer." ;
+          self#log#info "Not ready for a new track: empty buffer." ;
         r > 0
       end
 
@@ -119,7 +124,7 @@ object (self)
       let pos = Frame.position ab in
       buffering <- false ;
       if should_fail then begin
-        self#log#f 4 "Performing skip." ;
+        self#log#info "Performing skip." ;
         should_fail <- false ;
         if empty_on_abort then Generator.clear generator ;
         Frame.add_break ab (Frame.position ab)
@@ -133,9 +138,9 @@ object (self)
          * the start of a new track anyway, since the content after it
          * has nothing to do with the content before the connection). *)
         if Frame.is_partial ab then
-          self#log#f 4 "End of track." ;
+          self#log#info "End of track." ;
         if Generator.length generator = 0 then begin
-          self#log#f 4 "Buffer emptied, buffering needed." ;
+          self#log#info "Buffer emptied, buffering needed." ;
           buffering <- true
         end;
         if self#save_metadata ab && was_buffering && replay_meta then
