@@ -79,7 +79,7 @@ object (self)
               Bytes.get c 0
           in
             begin try
-              self#log#f 3 "Playing note %d." (note_of_char c);
+              self#log#important "Playing note %d." (note_of_char c);
               self#add_event 0 (MIDI.Note_on (note_of_char c, 0.8))
             with
               | Not_found -> ()
