@@ -42,7 +42,7 @@ object (self)
     let midi = MFrame.content buf (MFrame.position buf) in
     if chan >= Array.length midi then
       (
-        self#log#f 3 "Cannot read MIDI channel %d, stream only has %d channels." chan (Array.length midi);
+        self#log#important "Cannot read MIDI channel %d, stream only has %d channels." chan (Array.length midi);
         source#get buf
       )
     else
