@@ -1,9 +1,9 @@
-type json =
-  [ `Assoc of (string * json) list
+type t =
+  [ `Assoc of (string * t) list
   | `Bool of bool
   | `Float of float
   | `Int of int
-  | `List of json list
+  | `List of t list
   | `Null
   | `String of string
   ]
@@ -50,3 +50,24 @@ and parse_list stream =
 let from_string s =
   let lexer = lexer (Stream.of_string s) in
   parse lexer
+
+let escape_string s = String.escaped s
+
+let to_string (j:t) =
+  let blank indent = String.make (2*indent) ' ' in
+  let rec aux indent = function
+    | `Bool b -> if b then "true" else "false"
+    | `Int n -> string_of_int n
+    | `Float x -> string_of_float x
+    | `String s -> "\""^escape_string s^"\""
+    | `Null -> "null"
+    | `List l ->
+       let l = List.map (fun j -> blank (indent+1) ^ aux (indent+1) j) l in
+       let l = String.concat ",\n" l in
+       "[\n" ^ l ^ "\n" ^ blank indent ^ "]"
+    | `Assoc l ->
+       let l = List.map (fun (k,v) -> blank (indent+1) ^  "\"" ^ k ^ "\": " ^ aux (indent+1) v) l in
+       let l = String.concat ",\n" l in
+       "{\n" ^ l ^ "\n" ^ blank indent ^ "}"
+  in
+  aux 0 j
