@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2018 Savonet team
+  Copyright 2003-2019 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 
  *****************************************************************************)
 
@@ -54,9 +54,15 @@ let watch : File_watcher.watch = fun e file f ->
       end;
     let fd = Utils.get_some !fd in
     let event_conv = function
-      | `Modify -> Inotify.S_Modify
+      | `Modify ->
+          [Inotify.S_Modify; Inotify.S_Moved_to; Inotify.S_Moved_from;
+           Inotify.S_Delete; Inotify.S_Create]
     in
-    let wd = Inotify.add_watch fd file (List.map event_conv e) in
+    let e =
+      List.flatten
+        (List.map event_conv e)
+    in
+    let wd = Inotify.add_watch fd file e in
     handlers := (wd,f) :: !handlers;
     let unwatch =
       Tutils.mutexify m (fun () ->
