@@ -33,7 +33,9 @@ struct
     end;
     socket
 
-  let disconnect = Ssl.shutdown
+  let disconnect ssl =
+    Ssl.shutdown ssl;
+    Unix.close (Ssl.file_descr_of_socket ssl)
 
   let wait_for ?log event timeout =
     let event =
