@@ -98,19 +98,6 @@ let hashtbl_get : ('a,'b) Hashtbl.t -> 'a -> 'b option =
   fun h k ->
     try Some (Hashtbl.find h k) with Not_found -> None
 
-(* Backward-compatible API.. *)
-
-module LazyCompat =
-struct
-  let from_fun f = lazy (f ())
-  let from_val v = lazy v
-  (* Make compiler happy.. *)
-  let () =
-    ignore(from_fun (fun () -> ()));
-    ignore(from_val ())
-  include Lazy
-end
-
 (** Unescape a string. *)
 let unescape s =
   try
@@ -217,9 +204,9 @@ let read_all filename =
 let buffer_drop buffer len =
   let size = Buffer.length buffer in
   assert (len <= size) ;
-  if len = size then Buffer.reset buffer else
+  if len = size then Buffer.clear buffer else
     let tmp = Buffer.sub buffer len (size-len) in
-      Buffer.reset buffer ;
+      Buffer.clear buffer ;
       Buffer.add_string buffer tmp
 
 let unix_translator = 
@@ -338,6 +325,27 @@ let escape_string escape s =
   Format.pp_print_flush f () ;
   Buffer.contents b
 
+(** Remove line breaks from markdown text. This is useful for reflowing markdown such as when printing doc. *)
+let unbreak_md md =
+  let must_break = function
+    | ""::_ -> true
+    | "```"::_ -> true
+    | line::_ when line.[0] = '-' (* itemize *)-> true
+    | _ -> false
+  in
+  let rec text = function
+    | [] -> ""
+    | [line] -> line
+    | "```"::lines -> "```\n" ^ verb lines
+    | line::lines when line = "" || must_break lines -> line ^ "\n" ^ text lines
+    | line::lines -> line ^ " " ^ text lines
+  and verb = function
+    | "```"::lines -> "```\n" ^ text lines
+    | line::lines -> line ^ "\n" ^ verb lines
+    | [] -> "```"
+  in
+  text (String.split_on_char '\n' md)
+
 (* Here we take care not to introduce new redexes when substituting *)
 
 (* Interpolation:
@@ -393,6 +401,11 @@ let which ~path s =
   if test s then s else
     List.find test (List.map (fun d -> Filename.concat d s) path)
 
+let which_opt ~path s =
+  try
+    Some (which ~path s)
+  with Not_found -> None
+
 (** Get current timezone. *)
 external timezone : unit -> int = "liquidsoap_get_timezone"
 
@@ -463,7 +476,7 @@ let home_unrelate =
     unrel
 
 let get_tempdir () =
-  if Sys.os_type = "Win32" then
+  if Sys.win32 then
     getenv ~default:"C:\\temp" "TEMP"
   else
     getenv ~default:"/tmp" "TMPDIR"
@@ -655,10 +668,17 @@ let file_extension_len ~dir_sep name =
   in
   search_dot (String.length name - 1)
 
-let file_extension ?(dir_sep=Filename.dir_sep) name =
+let file_extension ?(leading_dot=true) ?(dir_sep=Filename.dir_sep) name =
   let dir_sep = dir_sep.[0] in
   let l = file_extension_len ~dir_sep name in
-  if l = 0 then "" else String.sub name (String.length name - l) l
+  let s =
+    if l = 0 then "" else
+      String.sub name (String.length name - l) l
+  in
+  try match leading_dot, s.[0] with
+    | false, '.' -> String.sub s 1 (String.length s - 1)
+    | _ -> s
+  with Invalid_argument _ -> s
 
 let quote s =
   let quote = '"' in
@@ -689,3 +709,18 @@ let environment () =
   in
   let l = Array.to_list l in
   List.map split l
+
+(** Size of an OCaml value (including referred elements) in bytes. *)
+let reachable_size x =
+  Obj.reachable_words (Obj.repr x) * Sys.word_size
+
+(** String representation of a size (in bytes). *)
+let string_of_size n =
+  if n < 1 lsl 10 then
+    Printf.sprintf "%d B" n
+  else if n < 1 lsl 20 then
+    Printf.sprintf "%.02f KiB" (float_of_int n /. float_of_int (1 lsl 10))
+  else if n < 1 lsl 30 then
+    Printf.sprintf "%.02f MiB" (float_of_int n /. float_of_int (1 lsl 20))
+  else
+    Printf.sprintf "%.02f GiB" (float_of_int n /. float_of_int (1 lsl 30))
